@@ -226,6 +226,7 @@ nitpick_ignore += [
     ('c:data', 'PyExc_AssertionError'),
     ('c:data', 'PyExc_AttributeError'),
     ('c:data', 'PyExc_BaseException'),
+    ('c:data', 'PyExc_BaseExceptionGroup'),
     ('c:data', 'PyExc_BlockingIOError'),
     ('c:data', 'PyExc_BrokenPipeError'),
     ('c:data', 'PyExc_BufferError'),
@@ -279,6 +280,7 @@ nitpick_ignore += [
     # C API: Standard Python warning classes
     ('c:data', 'PyExc_BytesWarning'),
     ('c:data', 'PyExc_DeprecationWarning'),
+    ('c:data', 'PyExc_EncodingWarning'),
     ('c:data', 'PyExc_FutureWarning'),
     ('c:data', 'PyExc_ImportWarning'),
     ('c:data', 'PyExc_PendingDeprecationWarning'),
@@ -300,7 +302,6 @@ nitpick_ignore += [
     ('py:attr', '__annotations__'),
     ('py:meth', '__missing__'),
     ('py:attr', '__wrapped__'),
-    ('py:attr', 'decimal.Context.clamp'),
     ('py:meth', 'index'),  # list.index, tuple.index, etc.
 ]
 
@@ -616,11 +617,19 @@ stable_abi_file = 'data/stable_abi.dat'
 # Options for sphinxext-opengraph
 # -------------------------------
 
-ogp_site_url = 'https://docs.python.org/3/'
+ogp_canonical_url = 'https://docs.python.org/3/'
 ogp_site_name = 'Python documentation'
-ogp_image = '_static/og-image.png'
+ogp_social_cards = {  # Used when matplotlib is installed
+    'image': '_static/og-image.png',
+    'line_color': '#3776ab',
+}
 ogp_custom_meta_tags = [
-    '<meta property="og:image:width" content="200">',
-    '<meta property="og:image:height" content="200">',
     '<meta name="theme-color" content="#3776ab">',
 ]
+if 'create-social-cards' not in tags:  # noqa: F821
+    # Define a static preview image when not creating social cards
+    ogp_image = '_static/og-image.png'
+    ogp_custom_meta_tags += [
+        '<meta property="og:image:width" content="200">',
+        '<meta property="og:image:height" content="200">',
+    ]
